﻿//////////////////////////////////////////////
// Entity.h	
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Mesh ;
	class Node ;
	class RenderQueue ;
	class Shader ;
	class SubEntity ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkExport
#include <NilkinsExport/Exportable/Exportable.h>

// Standards
#include <vector>

/// Class ------------------------------------

namespace nkGraphics
{	
	class DLL_GRAPHICS_EXPORT Entity final : public nkExport::Exportable
	{
		friend class RenderQueue ;

		private :
		
			// Attributes
			// Associated sub meshes
			std::vector<SubEntity*> _childrenArray ;

			// Parent node possible
			Node* _parentNode ;

			// Shaders attached
			Shader* _attachedShader ;
			Shader* _attachedRaytracingShader ;

			// Parent render queue
			RenderQueue* _renderQueue ;

		private :

			// Functions
			// Constructor, destructor
			Entity (RenderQueue* parentRenderQueue) ;
			~Entity () ;
		
		public :
		
			// Getters
			Node* getParentNode () const ;
			Shader* getShader () const ;
			Shader* getRaytracingShader () const ;
			RenderQueue* getRenderQueue () const ;

			// Setters
			void setParentNode (Node* parent) ;
			void setShader (Shader* shade) ;
			void setRaytracingShader (Shader* shade) ;

			// Utilities
			bool isCompatibleWith (Shader* shade) const ;
			void updateBasicShader (SubEntity* caller) ;
			Mesh* getMesh (int index) const ;

			// SubEnt maanagement
			SubEntity* addChild () ;
			SubEntity* getChild (unsigned int index) const ;
			int getChildCount () ;
			const std::vector<SubEntity*>& getChildren () const ;
			void removeChild (unsigned int index) ;
			void updateMeshBinding (SubEntity* caller, Mesh* oldMesh) ;			

			// Import / Export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}