﻿//////////////////////////////////////////////
// GlesBuffer.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

typedef unsigned int GLuint ;

/// Includes ---------------------------------

// nkGraphics
#include "../../../Buffers/Buffer.h"

/// Class ------------------------------------

namespace nkGraphics::gles
{
	class GlesBuffer final : public Buffer
	{
		private :

			// Attributes
			// GL data
			GLuint _id ;

			// Settings
			USAGE _usage ;
			BIND_FLAG _bindFlag ;
			CPU_ACCESS_FLAG _cpuAccessFlag ;

		public :

			// Constructor, destructor
			GlesBuffer () ;
			~GlesBuffer () ;

			// Getters
			virtual USAGE getUsage () const override ;
			virtual BIND_FLAG getBindFlag () const override ;
			virtual CPU_ACCESS_FLAG getCpuAccessFlag () const override ;
			virtual RESOURCE_MISC_FLAG getMiscFlag () const override ;
			GLuint getId () const ;

			// Setters
			virtual void setUsage (USAGE value) override ;
			virtual void setBindFlag (BIND_FLAG value) override ;
			virtual void setCpuAccessFlag (CPU_ACCESS_FLAG value) override ;
			virtual void setMiscFlag (RESOURCE_MISC_FLAG value) override ;
			virtual void copyParams (const Buffer& other) override ;

			// Loading
			virtual bool load () override ;
			virtual void unload () override ;

			// Activation
			virtual void activate (SHADER_TYPE shaderStage, unsigned int slot) override ;

			// Mapping
			virtual MapResult map (const MapRequestDescriptor& requestDesc) override ;
			virtual void unmap (const UnmapRequestDescriptor& requestDesc) override ;
			virtual void gpuCopyFrom (Buffer& other, const ResourceGpuCopyDescriptor& descriptor) override ;
	} ;
}