﻿//////////////////////////////////////////////
// Gl4FunctionRepository.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// Gl
#include <opengl/glcorearb.h>

/// Class ------------------------------------

namespace nkGraphics::gles
{
	// Buffers
	extern PFNGLBINDBUFFERPROC glBindBuffer ;
	extern PFNGLBINDBUFFERBASEPROC glBindBufferBase ;
	extern PFNGLBINDBUFFERRANGEPROC glBindBufferRange ;
	extern PFNGLBUFFERDATAPROC glBufferData ;
	extern PFNGLDELETEBUFFERSPROC glDeleteBuffers ;
	extern PFNGLGENBUFFERSPROC glGenBuffers ;

	// Clear
	extern PFNGLCLEARCOLORPROC glClearColor ;
	extern PFNGLCLEARPROC glClear ;

	// Draw
	extern PFNGLDRAWELEMENTSINSTANCEDPROC glDrawElementsInstanced ;

	// Framebuffers
	extern PFNGLBINDFRAMEBUFFERPROC glBindFramebuffer ;
	extern PFNGLDELETEFRAMEBUFFERSPROC glDeleteFramebuffers ;
	extern PFNGLFRAMEBUFFERTEXTURE2DPROC glFramebufferTexture2D ;
	extern PFNGLGENFRAMEBUFFERSPROC glGenFramebuffers ;

	// Global
	extern PFNGLDEBUGMESSAGECALLBACKPROC glDebugMessageCallback ;
	extern PFNGLDISABLEPROC glDisable ;
	extern PFNGLDISABLEIPROC glDisablei ;
	extern PFNGLENABLEPROC glEnable ;
	extern PFNGLENABLEIPROC glEnablei ;
	extern PFNGLFLUSHPROC glFlush ;
	extern PFNGLGETERRORPROC glGetError ;
	extern PFNGLGETSTRINGPROC glGetString ;
	extern PFNGLGETSTRINGIPROC glGetStringi ;

	// Meshes
	extern PFNGLDISABLEVERTEXATTRIBARRAYPROC glDisableVertexAttribArray ;
	extern PFNGLENABLEVERTEXATTRIBARRAYPROC glEnableVertexAttribArray ;
	extern PFNGLVERTEXATTRIBDIVISORPROC glVertexAttribDivisor ;
	extern PFNGLVERTEXATTRIBPOINTERPROC glVertexAttribPointer ;

	// Programs
	extern PFNGLATTACHSHADERPROC glAttachShader ;
	extern PFNGLCOMPILESHADERPROC glCompileShader ;
	extern PFNGLCREATEPROGRAMPROC glCreateProgram ;
	extern PFNGLCREATESHADERPROC glCreateShader ;
	extern PFNGLDELETEPROGRAMPROC glDeleteProgram ;
	extern PFNGLDELETESHADERPROC glDeleteShader ;
	extern PFNGLGETACTIVEUNIFORMPROC glGetActiveUniform ;
	extern PFNGLGETPROGRAMINFOLOGPROC glGetProgramInfoLog ;
	extern PFNGLGETPROGRAMIVPROC glGetProgramiv ;
	extern PFNGLGETSHADERINFOLOGPROC glGetShaderInfoLog ;
	extern PFNGLGETSHADERIVPROC glGetShaderiv ;
	extern PFNGLGETUNIFORMBLOCKINDEXPROC glGetUniformBlockIndex ;
	extern PFNGLGETUNIFORMLOCATIONPROC glGetUniformLocation ;
	extern PFNGLLINKPROGRAMPROC glLinkProgram ;
	extern PFNGLSHADERSOURCEPROC glShaderSource ;
	extern PFNGLUNIFORMBLOCKBINDINGPROC glUniformBlockBinding ;
	extern PFNGLUNIFORM1IPROC glUniform1i ;
	extern PFNGLUSEPROGRAMPROC glUseProgram ;

	// RenderStates
	// Blending
	extern PFNGLBLENDEQUATIONSEPARATEIPROC glBlendEquationSeparatei ;
	extern PFNGLBLENDFUNCSEPARATEIPROC glBlendFuncSeparatei ;
	// Depth
	extern PFNGLDEPTHFUNCPROC glDepthFunc ;
	extern PFNGLDEPTHMASKPROC glDepthMask ;
	// Raster
	extern PFNGLCULLFACEPROC glCullFace ;
	extern PFNGLFRONTFACEPROC glFrontFace ;

	// Samplers
	extern PFNGLBINDSAMPLERPROC glBindSampler ;
	extern PFNGLDELETESAMPLERSPROC glDeleteSamplers ;
	extern PFNGLGENSAMPLERSPROC glGenSamplers ;
	extern PFNGLSAMPLERPARAMETERFPROC glSamplerParameterf ;
	extern PFNGLSAMPLERPARAMETERIPROC glSamplerParameteri ;
	extern PFNGLSAMPLERPARAMETERFVPROC glSamplerParameterfv ;
	extern PFNGLSAMPLERPARAMETERIVPROC glSamplerParameteriv ;

	// Textures
	extern PFNGLACTIVETEXTUREPROC glActiveTexture ;
	extern PFNGLBINDTEXTUREPROC glBindTexture ;
	extern PFNGLDELETETEXTURESPROC glDeleteTextures ;
	extern PFNGLGENTEXTURESPROC glGenTextures ;
	extern PFNGLREADBUFFERPROC glReadBuffer ;
	extern PFNGLREADPIXELSPROC glReadPixels ;
	extern PFNGLTEXIMAGE2DPROC glTexImage2D ;
	extern PFNGLTEXPARAMETERIPROC glTexParameteri ;
	extern PFNGLTEXSTORAGE2DPROC glTexStorage2D ;

	// Viewports
	extern PFNGLVIEWPORTPROC glViewport ;

	// Utility class
	class GlesFunctionRepository final
	{
		public :

			// Loading
			static void loadGlFunctionsAddresses () ;
	} ;
}