﻿//////////////////////////////////////////////
// ViewportManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Viewport ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <unordered_map>
#include <mutex>
#include <string>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ViewportManager final : public nkCommon::SingletonClass<ViewportManager>
	{
		friend class nkCommon::SingletonClass<ViewportManager> ;

		private :

			// Attributs
			// Map des viewport en memoire
			std::unordered_map<std::string, Viewport*> _memory ;

			// Viewport actif
			Viewport* _activeViewport ;

			// Thread safe
			std::mutex _mutex ;

		private :

			// Functions
			// Constructeur, destructeur
			ViewportManager () ;

		public :

			// Destructeur
			~ViewportManager () ;

			// Getters
			Viewport* getActiveViewport () const ;

			// Setters
			void setActiveViewport (Viewport* value) ;

			// Manipulation memoire
			Viewport* createOrRetrieve (const std::string_view& name) ;
			Viewport* get (const std::string_view& name) ;
			Viewport* getByIndex (unsigned int index) ;
			void rename (const std::string_view& currentName, const std::string_view& newName) ;
			void erase (const std::string_view& name) ;			
	} ;
}