﻿//////////////////////////////////////////////
// IntVector.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkMaths
{
	class Vector ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/ByteAlignedClass.h>

/// Class ------------------------------------

namespace nkMaths
{
	class DLL_MATHS_EXPORT IntVector : public ByteAlignedClass<16>
	{
		public :
		
			int _x ;
			int _y ;
			int _z ;
			int _w ;
			
		public :
		
			// Constructor, destructor
			IntVector () ;
			IntVector (int x, int y) ;
			IntVector (int x, int y, int z) ;
			IntVector (int x, int y, int z, int w) ;
			IntVector (const Vector& other) ;
			IntVector (const IntVector& other) ;
			~IntVector () ;

			// Getters
			int getX () const ;
			int getY () const ;
			int getZ () const ;
			int getW () const ;

			// Setters
			void setX (int value) ;
			void setY (int value) ;
			void setZ (int value) ;
			void setW (int value) ;

			// Per component operations
			void addX (int value) ;
			void addY (int value) ;
			void addZ (int value) ;
			void addW (int value) ;
			void multX (int value) ;
			void multY (int value) ;
			void multZ (int value) ;
			void multW (int value) ;

			// Length
			float getLengthVec2 () const ;
			float getLengthVec3 () const ;
			float getLengthVec4 () const ;
			float getLengthSquaredVec2 () const ;
			float getLengthSquaredVec3 () const ;
			float getLengthSquaredVec4 () const ;
			// Distance
			float getDistanceVec2 (const IntVector& other) const ;
			float getDistanceVec3 (const IntVector& other) const ;
			float getDistanceVec4 (const IntVector& other) const ;
			float getDistanceSquaredVec2 (const IntVector& other) const ;
			float getDistanceSquaredVec3 (const IntVector& other) const ;
			float getDistanceSquaredVec4 (const IntVector& other) const ;

			// Utilities
			void fromString (const char* str) ;

			// Operators
			// Egalité
			IntVector& operator= (const IntVector& other) ;
			IntVector& operator= (const Vector& other) ;
			// Addition
			IntVector operator+ (const IntVector& other) const ;
			void operator+= (const IntVector& other) ;
			// Soustraction
			IntVector operator- (const IntVector& other) const ;
			void operator-= (const IntVector& other) ;
			// Multiplication
			IntVector operator* (const IntVector& other) const ;
			void operator*= (const IntVector& other) ;
			IntVector operator* (int coeff) const ;
			void operator*= (int coeff) ;
			// Division
			IntVector operator/ (const IntVector& other) const ;
			void operator/= (const IntVector& other) ;
			IntVector operator/ (int coeff) const ;
			void operator/= (int coeff) ;
			// Comparaison
			bool operator== (const IntVector& other) const ;
			bool operator!= (const IntVector& other) const ;
			bool operator< (const IntVector& other) const ;
			bool operator<= (const IntVector& other) const ;
			bool operator> (const IntVector& other) const ;
			bool operator>= (const IntVector& other) const ;
	} ;
}